import axios from 'axios';

// Sends an email using the BIC service

export default class Mail{

	static readonly address = 'definma@bosch-iot.com';  // Email address
	static uri: string;                                 // Mail API URI
	static auth = {username: '', password: ''};         // Mail API credentials
	static mailPass: string;                            // Mail API generates password

	static init() {
		if (process.env.NODE_ENV === 'production') {  // Only send mails in production
			this.mailPass = Array(64).fill(0).map(() => Math.floor(Math.random() * 10)).join('');
			this.uri = JSON.parse(process.env.VCAP_SERVICES).Mail[0].credentials.uri;
			this.auth.username = JSON.parse(process.env.VCAP_SERVICES).Mail[0].credentials.username;
			this.auth.password = JSON.parse(process.env.VCAP_SERVICES).Mail[0].credentials.password;
			axios({  // Get registered mail addresses
				method: 'get',
				url: this.uri + '/management/userDomainMapping',
				auth: this.auth
			}).then(res => {
				return new Promise(async (resolve, reject) => {
					try {
						if (res.data.addresses.indexOf(this.address) < 0) {  // Mail address not registered
							if (res.data.addresses.length) {  // Delete wrong registered mail address
								await axios({
									method: 'delete',
									url: this.uri + '/management/mailAddresses/' + res.data.addresses[0],
									auth: this.auth
								});
							}
							await axios({  // Register right mail address
								method: 'post',
								url: this.uri + '/management/mailAddresses/' + this.address,
								auth: this.auth
							});
						}
						resolve();
					}
					catch (e) {
						reject(e);
					}
				});
			}).then(() => {
				return axios({  // Set new mail password
					method: 'put',
					url: this.uri + '/management/mailAddresses/' + this.address + '/password/' + this.mailPass,
					auth: this.auth
				});
			}).then(() => {  // Init done successfully
				console.info('Mail service established successfully');
			}).catch(err => {  // Somewhere an error occurred
				console.error(`Mail init error: ${err.request.method} ${err.request.path}: ${err.response.status}`,
							  err.response.data);
			});
		}
	}

	static send (mailAddress, subject, content, f: (x?) => void = () => {}) {  // Callback executed empty or with error
		if (process.env.NODE_ENV === 'production') {  // Only send mails in production
			axios({
				method: 'post',
				url: this.uri + '/email',
				auth: this.auth,
				data: {
					recipients: [{to: mailAddress}],
					subject: {content: subject},
					body: {
						content: content,
						contentType: "text/html"
					},
					from: {
						eMail: this.address,
						password: this.mailPass
					}
				}
			}).then(() => {
				f();
			}).catch((err) => {
				f(err);
			});
		}
		else {  // Dev dummy replacement
			console.info('Sending mail to ' + mailAddress + ':  -- ' + subject + ' -- ' + content);
			f();
		}
	}
}
